@@ -191,12 +191,9 @@ void gfree_all( void )
 	gfree_names();
 	gfree_loadparm();
 	gfree_case_tables();
-	gfree_debugsyms();
 	gfree_charcnv();
 	gfree_interfaces();
-
-	/* release the talloc null_context memory last */
-	talloc_disable_null_tracking();
+	gfree_debugsyms();
 }
 
 const char *my_netbios_names(int i)
@@ -1038,6 +1035,7 @@ void become_daemon(bool Fork, bool no_process_group)
 }
 
 bool reinit_after_fork(struct messaging_context *msg_ctx,
+		       struct event_context *ev_ctx,
 		       bool parent_longlived)
 {
 	NTSTATUS status;
@@ -1054,15 +1052,21 @@ bool reinit_after_fork(struct messaging_context *msg_ctx,
 		return false;
 	}
 
-	/*
-	 * For clustering, we need to re-init our ctdbd connection after the
-	 * fork
-	 */
-	status = messaging_reinit(msg_ctx);
-	if (!NT_STATUS_IS_OK(status)) {
-		DEBUG(0,("messaging_reinit() failed: %s\n",
-			 nt_errstr(status)));
-		return false;
+	if (msg_ctx) {
+		/*
+		 * For clustering, we need to re-init our ctdbd connection after the
+		 * fork
+		 */
+		status = messaging_reinit(msg_ctx);
+		if (!NT_STATUS_IS_OK(status)) {
+			DEBUG(0,("messaging_reinit() failed: %s\n",
+				 nt_errstr(status)));
+			return false;
+		}
+	}
+
+	if (ev_ctx) {
+		event_context_reinit(ev_ctx);
 	}
 
 	return true;
