@@ -760,7 +760,7 @@ static char *get_kdc_ip_string(char *mem_ctx,
 		get_kdc_list(realm, sitename, &ip_srv_site, &count_site);
 
 		for (i = 0; i < count_site; i++) {
-			if (addr_equal(&ip_srv_site[i].ss, pss)) {
+			if (sockaddr_equal(&ip_srv_site[i].ss, pss)) {
 				continue;
 			}
 			/* Append to the string - inefficient
@@ -782,13 +782,13 @@ static char *get_kdc_ip_string(char *mem_ctx,
 	for (i = 0; i < count_nonsite; i++) {
 		int j;
 
-		if (addr_equal(&ip_srv_nonsite[i].ss, pss)) {
+		if (sockaddr_equal(&ip_srv_nonsite[i].ss, pss)) {
 			continue;
 		}
 
 		/* Ensure this isn't an IP already seen (YUK! this is n*n....) */
 		for (j = 0; j < count_site; j++) {
-			if (addr_equal(&ip_srv_nonsite[i].ss,
+			if (sockaddr_equal(&ip_srv_nonsite[i].ss,
 						&ip_srv_site[j].ss)) {
 				break;
 			}
@@ -881,9 +881,9 @@ bool create_local_private_krb5_conf_for_domain(const char *realm,
 
 	file_contents = talloc_asprintf(fname,
 					"[libdefaults]\n\tdefault_realm = %s\n"
-					"default_tgs_enctypes = RC4-HMAC DES-CBC-CRC DES-CBC-MD5\n"
-					"default_tkt_enctypes = RC4-HMAC DES-CBC-CRC DES-CBC-MD5\n"
-					"preferred_enctypes = RC4-HMAC DES-CBC-CRC DES-CBC-MD5\n\n"
+					"\tdefault_tgs_enctypes = RC4-HMAC DES-CBC-CRC DES-CBC-MD5\n"
+					"\tdefault_tkt_enctypes = RC4-HMAC DES-CBC-CRC DES-CBC-MD5\n"
+					"\tpreferred_enctypes = RC4-HMAC DES-CBC-CRC DES-CBC-MD5\n\n"
 					"[realms]\n\t%s = {\n"
 					"\t%s\t}\n",
 					realm_upper, realm_upper, kdc_ip_string);
