@@ -1067,13 +1067,10 @@ static void add_share_mode_entry(struct share_mode_lock *lck,
 }
 
 void set_share_mode(struct share_mode_lock *lck, files_struct *fsp,
-			uid_t uid, uint16 mid, uint16 op_type, bool initial_delete_on_close_allowed)
+		    uid_t uid, uint16 mid, uint16 op_type)
 {
 	struct share_mode_entry entry;
 	fill_share_mode_entry(&entry, fsp, uid, mid, op_type);
-	if (initial_delete_on_close_allowed) {
-		entry.flags |= SHARE_MODE_ALLOW_INITIAL_DELETE_ON_CLOSE;
-	}
 	add_share_mode_entry(lck, &entry);
 }
 
@@ -1189,7 +1186,16 @@ bool remove_share_oplock(struct share_mode_lock *lck, files_struct *fsp)
 	}
 
 	e->op_mid = 0;
-	e->op_type = NO_OPLOCK;
+	if (EXCLUSIVE_OPLOCK_TYPE(fsp->oplock_type)) {
+		/*
+		 * Going from exclusive or batch,
+ 		 * we always go through FAKE_LEVEL_II
+ 		 * first.
+ 		 */
+		e->op_type = FAKE_LEVEL_II_OPLOCK;
+	} else {
+		e->op_type = NO_OPLOCK;
+	}
 	lck->modified = True;
 	return True;
 }
@@ -1271,22 +1277,6 @@ NTSTATUS can_set_delete_on_close(files_struct *fsp, bool delete_on_close,
 	return NT_STATUS_OK;
 }
 
-/****************************************************************************
- Do we have an open file handle that created this entry ?
-****************************************************************************/
-
-bool can_set_initial_delete_on_close(const struct share_mode_lock *lck)
-{
-	int i;
-
-	for (i=0; i<lck->num_share_modes; i++) {
-		if (lck->share_modes[i].flags & SHARE_MODE_ALLOW_INITIAL_DELETE_ON_CLOSE) {
-			return True;
-		}
-	}
-	return False;
-}
-
 /*************************************************************************
  Return a talloced copy of a UNIX_USER_TOKEN. NULL on fail.
  (Should this be in locking.c.... ?).
@@ -1384,31 +1374,6 @@ bool set_delete_on_close(files_struct *fsp, bool delete_on_close, UNIX_USER_TOKE
 	return True;
 }
 
-/****************************************************************************
- Sets the allow initial delete on close flag for this share mode.
-****************************************************************************/
-
-bool set_allow_initial_delete_on_close(struct share_mode_lock *lck, files_struct *fsp, bool delete_on_close)
-{
-	struct share_mode_entry entry, *e;
-
-	/* Don't care about the pid owner being correct here - just a search. */
-	fill_share_mode_entry(&entry, fsp, (uid_t)-1, 0, NO_OPLOCK);
-
-	e = find_share_mode_entry(lck, &entry);
-	if (e == NULL) {
-		return False;
-	}
-
-	if (delete_on_close) {
-		e->flags |= SHARE_MODE_ALLOW_INITIAL_DELETE_ON_CLOSE;
-	} else {
-		e->flags &= ~SHARE_MODE_ALLOW_INITIAL_DELETE_ON_CLOSE;
-	}
-	lck->modified = True;
-	return True;
-}
-
 bool set_sticky_write_time(struct file_id fileid, struct timespec write_time)
 {
 	struct share_mode_lock *lck;
