@@ -657,18 +657,22 @@ static int vfswrap_ntimes(vfs_handle_struct *handle, const char *path, const str
 
 	START_PROFILE(syscall_ntimes);
 #if defined(HAVE_UTIMES)
-	{
+	if (ts != NULL) {
 		struct timeval tv[2];
 		tv[0] = convert_timespec_to_timeval(ts[0]);
 		tv[1] = convert_timespec_to_timeval(ts[1]);
 		result = utimes(path, tv);
+	} else {
+		result = utimes(path, NULL);
 	}
 #elif defined(HAVE_UTIME)
-	{
+	if (ts != NULL) {
 		struct utimbuf times;
 		times.actime = convert_timespec_to_time_t(ts[0]);
 		times.modtime = convert_timespec_to_time_t(ts[1]);
 		result = utime(path, times);
+	} else {
+		result = utime(path, NULL);
 	}
 #else
 	errno = ENOSYS;
@@ -710,6 +714,20 @@ static int strict_allocate_ftruncate(vfs_handle_struct *handle, files_struct *fs
 	if (st.st_size > len)
 		return sys_ftruncate(fsp->fh->fd, len);
 
+	/* available disk space is enough or not? */
+	if (lp_strict_allocate(SNUM(fsp->conn))){
+		SMB_BIG_UINT space_avail;
+		SMB_BIG_UINT bsize,dfree,dsize;
+
+		space_avail = get_dfree_info(fsp->conn,fsp->fsp_name,false,&bsize,&dfree,&dsize);
+		/* space_avail is 1k blocks */
+		if (space_avail == (SMB_BIG_UINT)-1 ||
+				((SMB_BIG_UINT)space_to_write/1024 > space_avail) ) {
+			errno = ENOSPC;
+			return -1;
+		}
+	}
+
 	/* Write out the real space on disk. */
 	if (SMB_VFS_LSEEK(fsp, st.st_size, SEEK_SET) != st.st_size)
 		return -1;
