@@ -207,28 +207,28 @@ static struct pdb_methods *pdb_get_methods(void)
 bool pdb_getsampwnam(struct samu *sam_acct, const char *username) 
 {
 	struct pdb_methods *pdb = pdb_get_methods();
-	struct samu *cache_copy;
+	struct samu *for_cache;
 	const struct dom_sid *user_sid;
 
 	if (!NT_STATUS_IS_OK(pdb->getsampwnam(pdb, sam_acct, username))) {
 		return False;
 	}
 
-	cache_copy = samu_new(NULL);
-	if (cache_copy == NULL) {
+	for_cache = samu_new(NULL);
+	if (for_cache == NULL) {
 		return False;
 	}
 
-	if (!pdb_copy_sam_account(cache_copy, sam_acct)) {
-		TALLOC_FREE(cache_copy);
+	if (!pdb_copy_sam_account(for_cache, sam_acct)) {
+		TALLOC_FREE(for_cache);
 		return False;
 	}
 
-	user_sid = pdb_get_user_sid(cache_copy);
+	user_sid = pdb_get_user_sid(for_cache);
 
 	memcache_add_talloc(NULL, PDB_GETPWSID_CACHE,
 			    data_blob_const(user_sid, sizeof(*user_sid)),
-			    cache_copy);
+			    &for_cache);
 
 	return True;
 }
@@ -1150,7 +1150,9 @@ static NTSTATUS pdb_default_rename_sam_account (struct pdb_methods *methods, str
 
 static NTSTATUS pdb_default_update_login_attempts (struct pdb_methods *methods, struct samu *newpwd, bool success)
 {
-	return NT_STATUS_NOT_IMPLEMENTED;
+	/* Only the pdb_nds backend implements this, by
+	 * default just return ok. */
+	return NT_STATUS_OK;
 }
 
 static NTSTATUS pdb_default_get_account_policy(struct pdb_methods *methods, int policy_index, uint32 *value)
