@@ -13,7 +13,7 @@
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
+   the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
@@ -22,8 +22,7 @@
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
  
 /*****************************************************/
 /*                                                   */
@@ -57,10 +56,10 @@
 
 /* Yes, these buggers are globals.... */
 const char *opt_requester_name = NULL;
-const char *opt_host = NULL; 
+const char *opt_host = NULL;
 const char *opt_password = NULL;
 const char *opt_user_name = NULL;
-BOOL opt_user_specified = False;
+bool opt_user_specified = False;
 const char *opt_workgroup = NULL;
 int opt_long_list_entries = 0;
 int opt_reboot = 0;
@@ -75,9 +74,9 @@ int opt_flags = -1;
 int opt_timeout = 0;
 const char *opt_target_workgroup = NULL;
 int opt_machine_pass = 0;
-BOOL opt_localgroup = False;
-BOOL opt_domaingroup = False;
-static BOOL do_talloc_report=False;
+int opt_localgroup = False;
+int opt_domaingroup = False;
+static int do_talloc_report=False;
 const char *opt_newntname = "";
 int opt_rid = 0;
 int opt_acls = 0;
@@ -85,12 +84,14 @@ int opt_attrs = 0;
 int opt_timestamps = 0;
 const char *opt_exclude = NULL;
 const char *opt_destination = NULL;
+int opt_testmode = False;
 
-BOOL opt_have_ip = False;
-struct in_addr opt_dest_ip;
+int opt_have_ip = False;
+struct sockaddr_storage opt_dest_ip;
+bool smb_encrypt;
+struct libnetapi_ctx *netapi_ctx = NULL;
 
-extern struct in_addr loopback_ip;
-extern BOOL AllowDebugChange;
+extern bool AllowDebugChange;
 
 uint32 get_sec_channel_type(const char *param) 
 {
@@ -158,79 +159,106 @@ int net_run_function2(int argc, const char **argv, const char *whoami,
 }
 
 /****************************************************************************
-connect to \\server\service 
+ Connect to \\server\service.
 ****************************************************************************/
 
-NTSTATUS connect_to_service(struct cli_state **c, struct in_addr *server_ip,
-					const char *server_name, 
-					const char *service_name, 
+NTSTATUS connect_to_service(struct cli_state **c,
+					struct sockaddr_storage *server_ss,
+					const char *server_name,
+					const char *service_name,
 					const char *service_type)
 {
 	NTSTATUS nt_status;
 
-	if (!opt_password && !opt_machine_pass) {
-		char *pass = getpass("Password:");
-		if (pass) {
-			opt_password = SMB_STRDUP(pass);
-		}
+	opt_password = net_prompt_pass(opt_user_name);
+	if (!opt_password) {
+		return NT_STATUS_NO_MEMORY;
 	}
 
-	nt_status = cli_full_connection(c, NULL, server_name, 
-					server_ip, opt_port,
-					service_name, service_type,  
+	nt_status = cli_full_connection(c, NULL, server_name,
+					server_ss, opt_port,
+					service_name, service_type,
 					opt_user_name, opt_workgroup,
 					opt_password, 0, Undefined, NULL);
+	if (!NT_STATUS_IS_OK(nt_status)) {
+		d_fprintf(stderr, "Could not connect to server %s\n", server_name);
 
-	if (NT_STATUS_IS_OK(nt_status) ||
-	    NT_STATUS_EQUAL(nt_status, NT_STATUS_NOLOGON_WORKSTATION_TRUST_ACCOUNT) ||
-	    NT_STATUS_EQUAL(nt_status, NT_STATUS_NOLOGON_SERVER_TRUST_ACCOUNT) ||
-	    NT_STATUS_EQUAL(nt_status, NT_STATUS_NOLOGON_INTERDOMAIN_TRUST_ACCOUNT)) {
+		/* Display a nicer message depending on the result */
+
+		if (NT_STATUS_V(nt_status) ==
+		    NT_STATUS_V(NT_STATUS_LOGON_FAILURE))
+			d_fprintf(stderr, "The username or password was not correct.\n");
+
+		if (NT_STATUS_V(nt_status) ==
+		    NT_STATUS_V(NT_STATUS_ACCOUNT_LOCKED_OUT))
+			d_fprintf(stderr, "The account was locked out.\n");
+
+		if (NT_STATUS_V(nt_status) ==
+		    NT_STATUS_V(NT_STATUS_ACCOUNT_DISABLED))
+			d_fprintf(stderr, "The account was disabled.\n");
 		return nt_status;
 	}
 
-	d_fprintf(stderr, "Could not connect to server %s\n", server_name);
+	if (smb_encrypt) {
+		nt_status = cli_force_encryption(*c,
+					opt_user_name,
+					opt_password,
+					opt_workgroup);
 
-	/* Display a nicer message depending on the result */
+		if (NT_STATUS_EQUAL(nt_status,NT_STATUS_NOT_SUPPORTED)) {
+			d_printf("Encryption required and "
+				"server that doesn't support "
+				"UNIX extensions - failing connect\n");
+		} else if (NT_STATUS_EQUAL(nt_status,NT_STATUS_UNKNOWN_REVISION)) {
+			d_printf("Encryption required and "
+				"can't get UNIX CIFS extensions "
+				"version from server.\n");
+		} else if (NT_STATUS_EQUAL(nt_status,NT_STATUS_UNSUPPORTED_COMPRESSION)) {
+			d_printf("Encryption required and "
+				"share %s doesn't support "
+				"encryption.\n", service_name);
+		} else if (!NT_STATUS_IS_OK(nt_status)) {
+			d_printf("Encryption required and "
+				"setup failed with error %s.\n",
+				nt_errstr(nt_status));
+		}
 
-	if (NT_STATUS_V(nt_status) ==
-	    NT_STATUS_V(NT_STATUS_LOGON_FAILURE))
-		d_fprintf(stderr, "The username or password was not correct.\n");
-
-	if (NT_STATUS_V(nt_status) ==
-	    NT_STATUS_V(NT_STATUS_ACCOUNT_LOCKED_OUT))
-		d_fprintf(stderr, "The account was locked out.\n");
-
-	if (NT_STATUS_V(nt_status) ==
-	    NT_STATUS_V(NT_STATUS_ACCOUNT_DISABLED))
-		d_fprintf(stderr, "The account was disabled.\n");
+		if (!NT_STATUS_IS_OK(nt_status)) {
+			cli_shutdown(*c);
+			*c = NULL;
+		}
+	}
 
 	return nt_status;
 }
 
-
 /****************************************************************************
-connect to \\server\ipc$  
+ Connect to \\server\ipc$.
 ****************************************************************************/
-NTSTATUS connect_to_ipc(struct cli_state **c, struct in_addr *server_ip,
-					const char *server_name)
+
+NTSTATUS connect_to_ipc(struct cli_state **c,
+			struct sockaddr_storage *server_ss,
+			const char *server_name)
 {
-	return connect_to_service(c, server_ip, server_name, "IPC$", "IPC");
+	return connect_to_service(c, server_ss, server_name, "IPC$", "IPC");
 }
 
 /****************************************************************************
-connect to \\server\ipc$ anonymously
+ Connect to \\server\ipc$ anonymously.
 ****************************************************************************/
+
 NTSTATUS connect_to_ipc_anonymous(struct cli_state **c,
-			struct in_addr *server_ip, const char *server_name)
+				struct sockaddr_storage *server_ss,
+				const char *server_name)
 {
 	NTSTATUS nt_status;
 
-	nt_status = cli_full_connection(c, opt_requester_name, server_name, 
-					server_ip, opt_port,
-					"IPC$", "IPC",  
+	nt_status = cli_full_connection(c, opt_requester_name, server_name,
+					server_ss, opt_port,
+					"IPC$", "IPC",
 					"", "",
 					"", 0, Undefined, NULL);
-	
+
 	if (NT_STATUS_IS_OK(nt_status)) {
 		return nt_status;
 	} else {
@@ -261,20 +289,19 @@ static char *get_user_and_realm(const char *username)
 }
 
 /****************************************************************************
-connect to \\server\ipc$ using KRB5
+ Connect to \\server\ipc$ using KRB5.
 ****************************************************************************/
 
 NTSTATUS connect_to_ipc_krb5(struct cli_state **c,
-			struct in_addr *server_ip, const char *server_name)
+			struct sockaddr_storage *server_ss,
+			const char *server_name)
 {
 	NTSTATUS nt_status;
 	char *user_and_realm = NULL;
 
-	if (!opt_password && !opt_machine_pass) {
-		char *pass = getpass("Password:");
-		if (pass) {
-			opt_password = SMB_STRDUP(pass);
-		}
+	opt_password = net_prompt_pass(opt_user_name);
+	if (!opt_password) {
+		return NT_STATUS_NO_MEMORY;
 	}
 
 	user_and_realm = get_user_and_realm(opt_user_name);
@@ -282,21 +309,33 @@ NTSTATUS connect_to_ipc_krb5(struct cli_state **c,
 		return NT_STATUS_NO_MEMORY;
 	}
 
-	nt_status = cli_full_connection(c, NULL, server_name, 
-					server_ip, opt_port,
-					"IPC$", "IPC",  
+	nt_status = cli_full_connection(c, NULL, server_name,
+					server_ss, opt_port,
+					"IPC$", "IPC",
 					user_and_realm, opt_workgroup,
 					opt_password, CLI_FULL_CONNECTION_USE_KERBEROS, 
 					Undefined, NULL);
-	
+
 	SAFE_FREE(user_and_realm);
 
-	if (NT_STATUS_IS_OK(nt_status)) {
-		return nt_status;
-	} else {
+	if (!NT_STATUS_IS_OK(nt_status)) {
 		DEBUG(1,("Cannot connect to server using kerberos.  Error was %s\n", nt_errstr(nt_status)));
 		return nt_status;
 	}
+
+        if (smb_encrypt) {
+		nt_status = cli_cm_force_encryption(*c,
+					user_and_realm,
+					opt_password,
+					opt_workgroup,
+                                        "IPC$");
+		if (!NT_STATUS_IS_OK(nt_status)) {
+			cli_shutdown(*c);
+			*c = NULL;
+		}
+	}
+
+	return nt_status;
 }
 
 /**
@@ -376,7 +415,7 @@ int net_use_krb_machine_account(void)
 int net_use_machine_account(void)
 {
 	char *user_name = NULL;
-		
+
 	if (!secrets_init()) {
 		d_fprintf(stderr, "ERROR: Unable to open secrets database\n");
 		exit(1);
@@ -390,66 +429,82 @@ int net_use_machine_account(void)
 	return 0;
 }
 
-BOOL net_find_server(const char *domain, unsigned flags, struct in_addr *server_ip, char **server_name)
+bool net_find_server(const char *domain,
+			unsigned flags,
+			struct sockaddr_storage *server_ss,
+			char **server_name)
 {
 	const char *d = domain ? domain : opt_target_workgroup;
 
 	if (opt_host) {
 		*server_name = SMB_STRDUP(opt_host);
-	}		
+	}
 
 	if (opt_have_ip) {
-		*server_ip = opt_dest_ip;
+		*server_ss = opt_dest_ip;
 		if (!*server_name) {
-			*server_name = SMB_STRDUP(inet_ntoa(opt_dest_ip));
+			char addr[INET6_ADDRSTRLEN];
+			print_sockaddr(addr, sizeof(addr), &opt_dest_ip);
+			*server_name = SMB_STRDUP(addr);
 		}
 	} else if (*server_name) {
 		/* resolve the IP address */
-		if (!resolve_name(*server_name, server_ip, 0x20))  {
+		if (!resolve_name(*server_name, server_ss, 0x20))  {
 			DEBUG(1,("Unable to resolve server name\n"));
-			return False;
+			return false;
 		}
 	} else if (flags & NET_FLAGS_PDC) {
-		struct in_addr pdc_ip;
+		fstring dc_name;
+		struct sockaddr_storage pdc_ss;
 
-		if (get_pdc_ip(d, &pdc_ip)) {
-			fstring dc_name;
-			
-			if (is_zero_ip(pdc_ip))
-				return False;
-			
-			if ( !name_status_find(d, 0x1b, 0x20, pdc_ip, dc_name) )
-				return False;
-				
-			*server_name = SMB_STRDUP(dc_name);
-			*server_ip = pdc_ip;
+		if (!get_pdc_ip(d, &pdc_ss)) {
+			DEBUG(1,("Unable to resolve PDC server address\n"));
+			return false;
 		}
-	} else if (flags & NET_FLAGS_DMB) {
-		struct in_addr msbrow_ip;
-		/*  if (!resolve_name(MSBROWSE, &msbrow_ip, 1)) */
-		if (!resolve_name(d, &msbrow_ip, 0x1B))  {
-			DEBUG(1,("Unable to resolve domain browser via name lookup\n"));
+
+		if (is_zero_addr(&pdc_ss)) {
+			return false;
+		}
+
+		if (!name_status_find(d, 0x1b, 0x20, &pdc_ss, dc_name)) {
 			return False;
-		} else {
-			*server_ip = msbrow_ip;
 		}
-		*server_name = SMB_STRDUP(inet_ntoa(opt_dest_ip));
+
+		*server_name = SMB_STRDUP(dc_name);
+		*server_ss = pdc_ss;
+	} else if (flags & NET_FLAGS_DMB) {
+		struct sockaddr_storage msbrow_ss;
+		char addr[INET6_ADDRSTRLEN];
+
+		/*  if (!resolve_name(MSBROWSE, &msbrow_ip, 1)) */
+		if (!resolve_name(d, &msbrow_ss, 0x1B))  {
+			DEBUG(1,("Unable to resolve domain browser via name lookup\n"));
+			return false;
+		}
+		*server_ss = msbrow_ss;
+		print_sockaddr(addr, sizeof(addr), server_ss);
+		*server_name = SMB_STRDUP(addr);
 	} else if (flags & NET_FLAGS_MASTER) {
-		struct in_addr brow_ips;
-		if (!resolve_name(d, &brow_ips, 0x1D))  {
+		struct sockaddr_storage brow_ss;
+		char addr[INET6_ADDRSTRLEN];
+		if (!resolve_name(d, &brow_ss, 0x1D))  {
 				/* go looking for workgroups */
 			DEBUG(1,("Unable to resolve master browser via name lookup\n"));
-			return False;
-		} else {
-			*server_ip = brow_ips;
+			return false;
 		}
-		*server_name = SMB_STRDUP(inet_ntoa(opt_dest_ip));
+		*server_ss = brow_ss;
+		print_sockaddr(addr, sizeof(addr), server_ss);
+		*server_name = SMB_STRDUP(addr);
 	} else if (!(flags & NET_FLAGS_LOCALHOST_DEFAULT_INSANE)) {
-		*server_ip = loopback_ip;
+		if (!interpret_string_addr(server_ss,
+					"127.0.0.1", AI_NUMERICHOST)) {
+			DEBUG(1,("Unable to resolve 127.0.0.1\n"));
+			return false;
+		}
 		*server_name = SMB_STRDUP("127.0.0.1");
 	}
 
-	if (!server_name || !*server_name) {
+	if (!*server_name) {
 		DEBUG(1,("no server to connect to\n"));
 		return False;
 	}
@@ -457,70 +512,72 @@ BOOL net_find_server(const char *domain, unsigned flags, struct in_addr *server_
 	return True;
 }
 
-
-BOOL net_find_pdc(struct in_addr *server_ip, fstring server_name, const char *domain_name)
+bool net_find_pdc(struct sockaddr_storage *server_ss,
+		fstring server_name,
+		const char *domain_name)
 {
-	if (get_pdc_ip(domain_name, server_ip)) {
-		if (is_zero_ip(*server_ip))
-			return False;
-		
-		if (!name_status_find(domain_name, 0x1b, 0x20, *server_ip, server_name))
-			return False;
-			
-		return True;	
-	} 
-	else
-		return False;
+	if (!get_pdc_ip(domain_name, server_ss)) {
+		return false;
+	}
+	if (is_zero_addr(server_ss)) {
+		return false;
+	}
+
+	if (!name_status_find(domain_name, 0x1b, 0x20, server_ss, server_name)) {
+		return false;
+	}
+
+	return true;
 }
 
-struct cli_state *net_make_ipc_connection( unsigned flags )
+NTSTATUS net_make_ipc_connection(unsigned flags, struct cli_state **pcli)
 {
-	return net_make_ipc_connection_ex( NULL, NULL, NULL, flags );
+	return net_make_ipc_connection_ex(NULL, NULL, NULL, flags, pcli);
 }
 
-struct cli_state *net_make_ipc_connection_ex( const char *domain, const char *server,
-                                              struct in_addr *ip, unsigned flags)
+NTSTATUS net_make_ipc_connection_ex(const char *domain, const char *server,
+                                    struct sockaddr_storage *pss, unsigned flags,
+				    struct cli_state **pcli)
 {
 	char *server_name = NULL;
-	struct in_addr server_ip;
+	struct sockaddr_storage server_ss;
 	struct cli_state *cli = NULL;
-	NTSTATUS nt_status = NT_STATUS_UNSUCCESSFUL;
+	NTSTATUS nt_status;
 
-	if ( !server || !ip ) {
-		if (!net_find_server(domain, flags, &server_ip, &server_name)) {
+	if ( !server || !pss ) {
+		if (!net_find_server(domain, flags, &server_ss, &server_name)) {
 			d_fprintf(stderr, "Unable to find a suitable server\n");
-			return NULL;
+			nt_status = NT_STATUS_UNSUCCESSFUL;
+			goto done;
 		}
 	} else {
 		server_name = SMB_STRDUP( server );
-		server_ip = *ip;
+		server_ss = *pss;
 	}
 
-	if (opt_user_name) {
-		nt_status = connect_to_ipc(&cli, &server_ip, server_name);
-		if (NT_STATUS_IS_OK(nt_status)) {
-			goto connected;
-		}
-	}
 	if (flags & NET_FLAGS_ANONYMOUS) {
-		nt_status = connect_to_ipc_anonymous(&cli, &server_ip, server_name);
-		if (NT_STATUS_IS_OK(nt_status)) {
-			goto connected;
-		}
+		nt_status = connect_to_ipc_anonymous(&cli, &server_ss, server_name);
+	} else {
+		nt_status = connect_to_ipc(&cli, &server_ss, server_name);
 	}
 
-	SAFE_FREE(server_name);
-	d_fprintf(stderr, "Connection failed: %s\n",
-		  nt_errstr(nt_status));
-	return NULL;
-
- connected:
 	/* store the server in the affinity cache if it was a PDC */
 
 	if ( (flags & NET_FLAGS_PDC) && NT_STATUS_IS_OK(nt_status) )
 		saf_store( cli->server_domain, cli->desthost );
 
-	return cli;
+	SAFE_FREE(server_name);
+	if (!NT_STATUS_IS_OK(nt_status)) {
+		d_fprintf(stderr, "Connection failed: %s\n",
+			  nt_errstr(nt_status));
+		cli = NULL;
+	}
+
+done:
+	if (pcli != NULL) {
+		*pcli = cli;
+	}
+	return nt_status;
 }
 
 static int net_user(int argc, const char **argv)
@@ -631,9 +688,9 @@ static int net_getlocalsid(int argc, const char **argv)
 		name = global_myname();
 	}
 
-	if(!initialize_password_db(False)) {
+	if(!initialize_password_db(False, NULL)) {
 		DEBUG(0, ("WARNING: Could not open passdb - local sid may not reflect passdb\n"
-			  "backend knowlege (such as the sid stored in LDAP)\n"));
+			  "backend knowledge (such as the sid stored in LDAP)\n"));
 	}
 
 	/* first check to see if we can even access secrets, so we don't
@@ -651,7 +708,7 @@ static int net_getlocalsid(int argc, const char **argv)
 		DEBUG(0, ("Can't fetch domain SID for name: %s\n", name));
 		return 1;
 	}
-	sid_to_string(sid_str, &sid);
+	sid_to_fstring(sid_str, &sid);
 	d_printf("SID for domain %s is: %s\n", name, sid_str);
 	return 0;
 }
@@ -701,18 +758,24 @@ static int net_getdomainsid(int argc, const char **argv)
 	DOM_SID domain_sid;
 	fstring sid_str;
 
-	if(!initialize_password_db(False)) {
-		DEBUG(0, ("WARNING: Could not open passdb - domain sid may not reflect passdb\n"
-			  "backend knowlege (such as the sid stored in LDAP)\n"));
+	if (argc > 0) {
+		d_printf("usage: net getdomainsid\n");
+		return 1;
+	}
+
+	if(!initialize_password_db(False, NULL)) {
+		DEBUG(0, ("WARNING: Could not open passdb - domain SID may "
+			  "not reflect passdb\n"
+			  "backend knowledge (such as the SID stored in "
+			  "LDAP)\n"));
 	}
 
 	/* first check to see if we can even access secrets, so we don't
 	   panic when we can't. */
 
 	if (!secrets_init()) {
-		d_fprintf(stderr, "Unable to open secrets.tdb.  "
-				  "Can't fetch domainSID for name: %s\n",
-				  get_global_sam_name());
+		d_fprintf(stderr, "Unable to open secrets.tdb.  Can't fetch domain"
+				  "SID for name: %s\n", get_global_sam_name());
 		return 1;
 	}
 
@@ -723,15 +786,15 @@ static int net_getdomainsid(int argc, const char **argv)
 		d_fprintf(stderr, "Could not fetch local SID\n");
 		return 1;
 	}
-	sid_to_string(sid_str, &domain_sid);
-	d_printf("SID for domain %s is: %s\n", global_myname(), sid_str);
+	sid_to_fstring(sid_str, &domain_sid);
+	d_printf("SID for local machine %s is: %s\n", global_myname(), sid_str);
 
 	if (!secrets_fetch_domain_sid(opt_workgroup, &domain_sid)) {
 		d_fprintf(stderr, "Could not fetch domain SID\n");
 		return 1;
 	}
 
-	sid_to_string(sid_str, &domain_sid);
+	sid_to_fstring(sid_str, &domain_sid);
 	d_printf("SID for domain %s is: %s\n", opt_workgroup, sid_str);
 
 	return 0;
@@ -819,7 +882,7 @@ static int net_afs(int argc, const char **argv)
 
 #endif /* WITH_FAKE_KASERVER */
 
-static BOOL search_maxrid(struct pdb_search *search, const char *type,
+static bool search_maxrid(struct pdb_search *search, const char *type,
 			  uint32 *max_rid)
 {
 	struct samr_displayentry *entries;
@@ -873,6 +936,33 @@ static int net_maxrid(int argc, const char **argv)
 	return 0;
 }
 
+/****************************************************************************
+****************************************************************************/
+
+const char *net_prompt_pass(const char *user)
+{
+	char *prompt = NULL;
+	const char *pass = NULL;
+
+	if (opt_password) {
+		return opt_password;
+	}
+
+	if (opt_machine_pass) {
+		return NULL;
+	}
+
+	asprintf(&prompt, "Enter %s's password:", user);
+	if (!prompt) {
+		return NULL;
+	}
+
+	pass = getpass(prompt);
+	SAFE_FREE(prompt);
+
+	return pass;
+}
+
 /* main function table */
 static struct functable net_func[] = {
 	{"RPC", net_rpc},
@@ -900,6 +990,7 @@ static struct functable net_func[] = {
 	{"TIME", net_time},
 	{"LOOKUP", net_lookup},
 	{"JOIN", net_join},
+	{"DOM", net_dom},
 	{"CACHE", net_cache},
 	{"GETLOCALSID", net_getlocalsid},
 	{"SETLOCALSID", net_setlocalsid},
@@ -910,6 +1001,8 @@ static struct functable net_func[] = {
 	{"STATUS", net_status},
 	{"USERSHARE", net_usershare},
 	{"USERSIDLIST", net_usersidlist},
+	{"CONF", net_conf},
+	{"REGISTRY", net_registry},
 #ifdef WITH_FAKE_KASERVER
 	{"AFS", net_afs},
 #endif
@@ -939,6 +1032,7 @@ static struct functable net_func[] = {
 		{"port",	'p', POPT_ARG_INT,    &opt_port},
 		{"myname",	'n', POPT_ARG_STRING, &opt_requester_name},
 		{"server",	'S', POPT_ARG_STRING, &opt_host},
+		{"encrypt",	'e', POPT_ARG_NONE,   NULL, 'e', "Encrypt SMB transport (UNIX extended servers only)" },
 		{"container",	'c', POPT_ARG_STRING, &opt_container},
 		{"comment",	'C', POPT_ARG_STRING, &opt_comment},
 		{"maxusers",	'M', POPT_ARG_INT,    &opt_maxusers},
@@ -951,6 +1045,7 @@ static struct functable net_func[] = {
 		{"machine-pass",'P', POPT_ARG_NONE,   &opt_machine_pass},
 		{"myworkgroup", 'W', POPT_ARG_STRING, &opt_workgroup},
 		{"verbose",	'v', POPT_ARG_NONE,   &opt_verbose},
+		{"test",	'T', POPT_ARG_NONE,   &opt_testmode},
 		/* Options for 'net groupmap set' */
 		{"local",       'L', POPT_ARG_NONE,   &opt_localgroup},
 		{"domain",      'D', POPT_ARG_NONE,   &opt_domaingroup},
@@ -960,7 +1055,7 @@ static struct functable net_func[] = {
 		{"acls",	0, POPT_ARG_NONE,     &opt_acls},
 		{"attrs",	0, POPT_ARG_NONE,     &opt_attrs},
 		{"timestamps",	0, POPT_ARG_NONE,     &opt_timestamps},
-		{"exclude",	'e', POPT_ARG_STRING, &opt_exclude},
+		{"exclude",	'X', POPT_ARG_STRING, &opt_exclude},
 		{"destination",	0, POPT_ARG_STRING,   &opt_destination},
 		{"tallocreport", 0, POPT_ARG_NONE, &do_talloc_report},
 
@@ -968,7 +1063,9 @@ static struct functable net_func[] = {
 		{ 0, 0, 0, 0}
 	};
 
-	zero_ip(&opt_dest_ip);
+	TALLOC_CTX *frame = talloc_stackframe();
+
+	zero_sockaddr(&opt_dest_ip);
 
 	load_case_tables();
 
@@ -985,12 +1082,16 @@ static struct functable net_func[] = {
 			net_help(argc, argv);
 			exit(0);
 			break;
+		case 'e':
+			smb_encrypt=true;
+			break;
 		case 'I':
-			opt_dest_ip = *interpret_addr2(poptGetOptArg(pc));
-			if (is_zero_ip(opt_dest_ip))
+			if (!interpret_string_addr(&opt_dest_ip,
+						poptGetOptArg(pc), 0)) {
 				d_fprintf(stderr, "\nInvalid ip address specified\n");
-			else
+			} else {
 				opt_have_ip = True;
+			}
 			break;
 		case 'U':
 			opt_user_specified = True;
@@ -1014,7 +1115,7 @@ static struct functable net_func[] = {
 	 * set by cmdline arg or remain default (0)
 	 */
 	AllowDebugChange = False;
-	lp_load(dyn_CONFIGFILE,True,False,False,True);
+	lp_load(get_dyn_CONFIGFILE(),True,False,False,True);
 	
  	argv_new = (const char **)poptGetArgs(pc);
 
@@ -1038,10 +1139,6 @@ static struct functable net_func[] = {
 		opt_user_name = getenv("LOGNAME");
 	}
 
-	if (!opt_user_name) {
-		opt_user_name = "";
-	}
-
 	if (!opt_workgroup) {
 		opt_workgroup = smb_xstrdup(lp_workgroup());
 	}
@@ -1073,5 +1170,9 @@ static struct functable net_func[] = {
 	rc = net_run_function(argc_new-1, argv_new+1, net_func, net_help);
 	
 	DEBUG(2,("return code = %d\n", rc));
+
+	libnetapi_free(netapi_ctx);
+
+	TALLOC_FREE(frame);
 	return rc;
 }
