@@ -170,35 +170,35 @@ static void terminate(bool is_parent)
 	exit(0);
 }
 
-static bool do_sigterm;
+static SIG_ATOMIC_T do_sigterm = 0;
 
 static void termination_handler(int signum)
 {
-	do_sigterm = True;
+	do_sigterm = 1;
 	sys_select_signal(signum);
 }
 
-static bool do_sigusr2;
+static SIG_ATOMIC_T do_sigusr2 = 0;
 
 static void sigusr2_handler(int signum)
 {
-	do_sigusr2 = True;
+	do_sigusr2 = 1;
 	sys_select_signal(SIGUSR2);
 }
 
-static bool do_sighup;
+static SIG_ATOMIC_T do_sighup = 0;
 
 static void sighup_handler(int signum)
 {
-	do_sighup = True;
+	do_sighup = 1;
 	sys_select_signal(SIGHUP);
 }
 
-static bool do_sigchld;
+static SIG_ATOMIC_T do_sigchld = 0;
 
 static void sigchld_handler(int signum)
 {
-	do_sigchld = True;
+	do_sigchld = 1;
 	sys_select_signal(SIGCHLD);
 }
 
@@ -221,7 +221,7 @@ static void msg_shutdown(struct messaging_context *msg,
 			 struct server_id server_id,
 			 DATA_BLOB *data)
 {
-	do_sigterm = True;
+	do_sigterm = 1;
 }
 
 
@@ -746,6 +746,7 @@ static void new_connection(int listen_sock, bool privileged)
 static void remove_client(struct winbindd_cli_state *state)
 {
 	char c = 0;
+	int nwritten;
 
 	/* It's a dead client - hold a funeral */
 	
@@ -754,7 +755,11 @@ static void remove_client(struct winbindd_cli_state *state)
 	}
 
 	/* tell client, we are closing ... */
-	write(state->sock, &c, sizeof(c));
+	nwritten = write(state->sock, &c, sizeof(c));
+	if (nwritten == -1) {
+		DEBUG(2, ("final write to client failed: %s\n",
+			  strerror(errno)));
+	}
 
 	/* Close socket */
 		
@@ -822,7 +827,7 @@ void winbind_check_sighup(const char *logfile)
 		flush_caches();
 		reload_services_file(logfile);
 
-		do_sighup = False;
+		do_sighup = 0;
 	}
 }
 
@@ -997,13 +1002,13 @@ static void process_loop(void)
 
 	if (do_sigusr2) {
 		print_winbindd_status();
-		do_sigusr2 = False;
+		do_sigusr2 = 0;
 	}
 
 	if (do_sigchld) {
 		pid_t pid;
 
-		do_sigchld = False;
+		do_sigchld = 0;
 
 		while ((pid = sys_waitpid(-1, NULL, WNOHANG)) > 0) {
 			winbind_child_died(pid);
@@ -1217,7 +1222,13 @@ int main(int argc, char **argv, char **envp)
 
 	TimeInit();
 
-	if (!reinit_after_fork(winbind_messaging_context(), false)) {
+	/* Don't use winbindd_reinit_after_fork here as
+	 * we're just starting up and haven't created any
+	 * winbindd-specific resources we must free yet. JRA.
+	 */
+
+	if (!reinit_after_fork(winbind_messaging_context(),
+			       winbind_event_context(), false)) {
 		DEBUG(0,("reinit_after_fork() failed\n"));
 		exit(1);
 	}
